
#include "I2Cdev.h"

#include "MPU6050_6Axis_MotionApps20.h"

#if I2CDEV_IMPLEMENTATION == I2CDEV_ARDUINO_WIRE
    #include "Wire.h"
#endif

#include <Servo.h>
#include <SPI.h>


MPU6050 mpu;

const int csPinX = 4;
const int csPinY = 5;
const int buttonPin = 12;
int CalVal_Yaw = 0;
int CalVal_Pitch = 0;
int Calibrate = 0;
int Rotation_Yaw = 0;
int Rotation_Pitch = 0;
int ReadIn = A0;
int ReadValue = 0;
int buttonState = 0; 


#define OUTPUT_READABLE_YAWPITCHROLL

#define INTERRUPT_PIN 2  // use pin 2 on Arduino Uno & most boards
#define LED_PIN 13 // (Arduino is 13, Teensy is 11, Teensy++ is 6)
bool blinkState = false;

// MPU control/status vars
bool dmpReady = false;  // set true if DMP init was successful
uint8_t mpuIntStatus;   // holds actual interrupt status byte from MPU
uint8_t devStatus;      // return status after each device operation (0 = success, !0 = error)
uint16_t packetSize;    // expected DMP packet size (default is 42 bytes)
uint16_t fifoCount;     // count of all bytes currently in FIFO
uint8_t fifoBuffer[64]; // FIFO storage buffer

// orientation/motion vars
Quaternion q;           // [w, x, y, z]         quaternion container
VectorInt16 aa;         // [x, y, z]            accel sensor measurements
VectorInt16 aaReal;     // [x, y, z]            gravity-free accel sensor measurements
VectorInt16 aaWorld;    // [x, y, z]            world-frame accel sensor measurements
VectorFloat gravity;    // [x, y, z]            gravity vector
float euler[3];         // [psi, theta, phi]    Euler angle container
float ypr[3];           // [yaw, pitch, roll]   yaw/pitch/roll container and gravity vector

// packet structure for InvenSense teapot demo
uint8_t teapotPacket[14] = { '$', 0x02, 0,0, 0,0, 0,0, 0,0, 0x00, 0x00, '\r', '\n' };



// ================================================================
// ===               INTERRUPT DETECTION ROUTINE                ===
// ================================================================

volatile bool mpuInterrupt = false;     // indicates whether MPU interrupt pin has gone high
void dmpDataReady() {
    mpuInterrupt = true;
}
 

// ==================================================================================================================================================
// ===                                                              INITIAL SETUP                                                                 ===
// ==================================================================================================================================================

void setup() {

    pinMode(csPinX, OUTPUT);
    pinMode(csPinY, OUTPUT);
    pinMode(buttonPin, INPUT_PULLUP);
    SPI.begin();
   
     
    // join I2C bus (I2Cdev library doesn't do this automatically)
    #if I2CDEV_IMPLEMENTATION == I2CDEV_ARDUINO_WIRE
        Wire.begin();
        Wire.setClock(400000); // 400kHz I2C clock. Comment this line if having compilation difficulties
    #elif I2CDEV_IMPLEMENTATION == I2CDEV_BUILTIN_FASTWIRE
        Fastwire::setup(400, true);
    #endif

    // initialize serial communication
    // (115200 chosen because it is required for Teapot Demo output, but it's
    // really up to you depending on your project)
    Serial.begin(115200);
    while (!Serial); // wait for Leonardo enumeration, others continue immediately

    // initialize device
    Serial.println(F("Initializing I2C devices..."));
    mpu.initialize();
    pinMode(INTERRUPT_PIN, INPUT);

    // verify connection
    Serial.println(F("Testing device connections..."));
    Serial.println(mpu.testConnection() ? F("MPU6050 connection successful") : F("MPU6050 connection failed"));

    // wait for ready
    Serial.println(F("\nSend any character to begin DMP programming and demo: "));
    while (Serial.available() && Serial.read()); // empty buffer
    //while (!Serial.available());                 // wait for data
    while (Serial.available() && Serial.read()); // empty buffer again

    // load and configure the DMP
    Serial.println(F("Initializing DMP..."));
    devStatus = mpu.dmpInitialize();

    // supply your own gyro offsets here, scaled for min sensitivity
    mpu.setXGyroOffset(129);
    mpu.setYGyroOffset(-79);
    mpu.setZGyroOffset(27);
    mpu.setZAccelOffset(1987); // 1688 factory default for my test chip

    // make sure it worked (returns 0 if so)
    if (devStatus == 0) {

        // turn on the DMP, now that it's ready
        Serial.println(F("Enabling DMP..."));
        mpu.setDMPEnabled(true);

        // enable Arduino interrupt detection
        Serial.println(F("Enabling interrupt detection (Arduino external interrupt 0)..."));
        attachInterrupt(digitalPinToInterrupt(INTERRUPT_PIN), dmpDataReady, RISING);
        mpuIntStatus = mpu.getIntStatus();

        // set our DMP Ready flag so the main loop() function knows it's okay to use it
        Serial.println(F("DMP ready! Waiting for first interrupt..."));
        dmpReady = true;

        // get expected DMP packet size for later comparison
        packetSize = mpu.dmpGetFIFOPacketSize();
    } else {
        // ERROR!
        // 1 = initial memory load failed
        // 2 = DMP configuration updates failed
        // (if it's going to break, usually the code will be 1)
        Serial.print(F("DMP Initialization failed (code "));
        Serial.print(devStatus);
        Serial.println(F(")"));
    }

    
    // configure LED for output
    pinMode(LED_PIN, OUTPUT);
}

// ==================================================================================================================================================
// ===                                                               MAIN PROGRAM LOOP                                                            ===
// ==================================================================================================================================================

void loop() {
    // if programming failed, don't try to do anything
    if (!dmpReady) return;

    // wait for MPU interrupt or extra packet(s) available
    while (!mpuInterrupt && fifoCount < packetSize) {
        // other program behavior stuff here
    }

    // reset interrupt flag and get INT_STATUS byte
    mpuInterrupt = false;
    mpuIntStatus = mpu.getIntStatus();
 
    // get current FIFO count
    fifoCount = mpu.getFIFOCount();

    // check for overflow (this should never happen unless our code is too inefficient)
    if ((mpuIntStatus & 0x10) || fifoCount == 1024) {
        // reset so we can continue cleanly
        mpu.resetFIFO();
        Serial.println(F("FIFO overflow!"));

    // otherwise, check for DMP data ready interrupt (this should happen frequently)
    } else if (mpuIntStatus & 0x02) {
        // wait for correct available data length, should be a VERY short wait
        while (fifoCount < packetSize) fifoCount = mpu.getFIFOCount();

        // read a packet from FIFO
        mpu.getFIFOBytes(fifoBuffer, packetSize);
        
        // track FIFO count here in case there is > 1 packet available
        // (this lets us immediately read more without waiting for an interrupt)
        fifoCount -= packetSize;

// ================================================================
// ===                     Calibration                          ===
// ================================================================

  buttonState = digitalRead(buttonPin);

  if (buttonState == LOW) {
     Calibrate = 0;
  } 
 
        if (Calibrate < 4){
          CalVal_Yaw = (ypr[0] * 180/M_PI +180);
          CalVal_Pitch = (ypr[1] * 180/M_PI +180);
          Calibrate++;
        }

// ================================================================
// ===                    MPU->DigitalPot                       ===
// ================================================================

        #ifdef OUTPUT_READABLE_YAWPITCHROLL
            // display Euler angles in degrees
            mpu.dmpGetQuaternion(&q, fifoBuffer);
            mpu.dmpGetGravity(&gravity, &q);
            mpu.dmpGetYawPitchRoll(ypr, &q, &gravity);
            //Serial.print("ypr\t");

            Serial.print("Raw Value Y/P: ");
            Serial.print((ypr[0] * 180/M_PI)+180);
            Serial.print(" / ");
            Serial.print(ypr[1] * 180/M_PI+180);
            Serial.print("\t");
            Serial.print("\t");
            
            Rotation_Yaw = (ypr[0] * 180/M_PI+180) - CalVal_Yaw + 90;
            Rotation_Pitch = (ypr[1] * 180/M_PI+180) - CalVal_Pitch + 45;

            
            if (Rotation_Yaw > 128){
               Rotation_Yaw = 128;
            }

            if (Rotation_Yaw < 0){
               Rotation_Yaw = 0;
            }

           /*if (Rotation_Yaw > 31 && Rotation_Yaw < 150) {
              Rotation_Yaw = Rotation_Yaw+70;
            }*/
            

            if (Rotation_Pitch > 128){
               Rotation_Pitch = 128;
            }

            if (Rotation_Pitch < 0){
               Rotation_Pitch = 0;
            }
            
            if (Rotation_Pitch > 31 && Rotation_Pitch < 150) {
              Rotation_Pitch = Rotation_Pitch+10;
            }

           
            digitalWrite(4, LOW);
            SPI.transfer( 0);
            SPI.transfer(Rotation_Yaw);
            digitalWrite(4, HIGH);
            
            digitalWrite(5, LOW);
            SPI.transfer( 0);
            SPI.transfer(Rotation_Pitch);
            digitalWrite(5, HIGH);

        #endif     
  

        // blink LED to indicate activity
        blinkState = !blinkState;
        digitalWrite(LED_PIN, blinkState);
    }
}
